﻿using System;
using System.IO;
using System.Threading;
using System.Reflection;
using System.Diagnostics;
using System.Text.RegularExpressions;
using System.Windows.Forms;

namespace WinCycle
{
    public partial class SplashScreen : Form
    {
        delegate void SplashShowCloseDelegate();
        static SplashScreen frmSplashScreen = null;

        public static void Start()
        {
            if (frmSplashScreen == null)
            {
                frmSplashScreen = new SplashScreen();
                frmSplashScreen.ShowSplashScreen();
            }
        }

        public static void Stop()
        {
            if (frmSplashScreen != null)
            {
                frmSplashScreen.CloseSplashScreen();
                frmSplashScreen = null;
            }
        }

        public void ShowSplashScreen()
        {
            if (InvokeRequired)
            {   // We're not in the UI thread, so we need to call BeginInvoke
                BeginInvoke(new SplashShowCloseDelegate(ShowSplashScreen));
                return;
            }
            this.Show();
            Application.Run(this);
        }

        public void CloseSplashScreen()
        {
            if (InvokeRequired)
            {   // We're not in the UI thread, so we need to call BeginInvoke
                BeginInvoke(new SplashShowCloseDelegate(CloseSplashScreen));
                return;
            }
            this.Close();
        }

        public SplashScreen()
        {
            InitializeComponent();
            //this.ClientSize = this.BackgroundImage.Size;
            string longVersion = Assembly.GetExecutingAssembly().GetName().Version.ToString();
            string shortVersion = new Regex(@"^\d+\.\d+").Match(longVersion).Value;
            lblAbtProductName.Text = Application.ProductName;
            lblAbtFullName.Text = Win32andMore.GetDescription();
            string buildDate = RetrieveLinkerTimestamp().ToShortDateString();
            //string buildDate = new FileInfo(Assembly.GetExecutingAssembly().Location).LastWriteTime.ToShortDateString();
            lblAbtVersion.Text = "Version " + longVersion + " (" + buildDate + ")";
            //this.Opacity = 0;
            tmrFadeIn.Enabled = true;
        }

        private DateTime RetrieveLinkerTimestamp() // lässt sich nicht in Utilities.cs verlagern
        {
            string filePath = Assembly.GetCallingAssembly().Location;
            const int c_PeHeaderOffset = 60;
            const int c_LinkerTimestampOffset = 8;
            byte[] b = new byte[2048];
            Stream s = null;
            try
            {
                s = new FileStream(filePath, FileMode.Open, FileAccess.Read);
                s.Read(b, 0, 2048);
            }
            finally { if (s != null) s.Close(); }
            int i = BitConverter.ToInt32(b, c_PeHeaderOffset);
            int secondsSince1970 = BitConverter.ToInt32(b, i + c_LinkerTimestampOffset);
            DateTime dt = new DateTime(1970, 1, 1, 0, 0, 0);
            dt = dt.AddSeconds(secondsSince1970);
            dt = dt.AddHours(TimeZone.CurrentTimeZone.GetUtcOffset(dt).Hours);
            return dt;
        }

        private void FormAbout_KeyDown(object sender, KeyEventArgs e)
        {
            if (e.KeyCode == Keys.Escape) CloseSplashScreen(); //this.Close(); // Formular schließen
        }

        private void FormAbout_Click(object sender, EventArgs e)
        {
            CloseSplashScreen();
            this.Close();
        }

        private void lblAbtLink_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            try { Process.Start("http://www.ophthalmostar.de/molistar.htm"); }
            catch (Exception ex) { MessageBox.Show(ex.Message, lblAbtVersion.Text); }
        }

        private void lblAbtVersion_Click(object sender, EventArgs e) { this.Close(); }
        private void lblAbtCopyright_Click(object sender, EventArgs e) { this.Close(); }
        private void lblAbtSubCaption_Click(object sender, EventArgs e) { this.Close(); }
        private void lblAbtFullName_Click(object sender, EventArgs e) { this.Close(); }

        private void tmrFadeIn_Tick(object sender, EventArgs e)
        {
            this.Opacity += 0.07;
            if (this.Opacity >= .95)
            {
                this.Opacity = 1;
                tmrFadeIn.Enabled = false;
            }
        }

        private void SplashScreenForm_FormClosing(object sender, FormClosingEventArgs e)
        {
            Application.DoEvents(); // verhindert Flackern
            while (this.Opacity >= .2)
            {
                this.Opacity -= 0.09;
                this.Refresh(); // komischerweise sonst kein Fadeout
                Thread.Sleep(15);
            }
        }

    }
}
